//*****************************************************************//
// Name    : VR MAZE Implementation - Requires Completion of Lab 6 //
// Author  : Mark Huffman                                          //
// Date    : 11/27/17                                              //
// Version : 2.0                                                   //
//*****************************************************************//

/////////////////////////////////
// GENERAL MAIN ARDUINO SETUP //
////////////////////////////////
// Add the lines after each note to its desinated location within your main file

/*================================
*//* Includes (Add before setup())
==================================
#include <SoftwareSerial.h>

===============================
*//* Setup (Add beofre setup())
===============================
SoftwareSerial blueTooth(txPin, rxPin);
static boolean start = true;

=====================================================
*//* Bluetoothup/Unity Configuration (Add to setup())
=====================================================
  blueTooth.begin(9600);
  connectUnity();
  int times[] = {#,#,#,#,#,#}; // Set timming values!
  setupUnity(times);
  
============================================================================
*//* First Movement (Add to loop() before everything else after "static MyRobot")
=============================================================================
if(start){
    // Start Motors
    unityUpdate(countTillNextTurn(mazeBot)); // Pass "MyRobot"
    motorControl("Go"); // Your Start MOTORS command
    start = false;
    
  }else{
    // the rest of loop()
    
================================================
*//* Before starting Everyturn (After Stopping!)
================================================
unityUpdate(getNextTurnAction(mazeBot)); // Pass "MyRobot"

=================================================
*//* After completing Everyturn (After Stopping!)
=================================================
unityUpdate(countTillNextTurn(mazeBot)); // Pass "MyRobot"

  
*/////////////////////////////////
// Unity Communication Commands //
//////////////////////////////////

void checkUnity(){
  // Responds to all Unity requests as "Bot" not ready
  // Keeps connection live
  if (blueTooth.available())  {
    // Read the incomming signal
    char c = blueTooth.read();
    if(c == 'R') {
      // Not Ready to send next instuction
      blueTooth.println("W"); // tell unity to wait
    }
  }
}

void connectUnity(){
  // Holds here till unity replies with a connection signal
  // Note* Bot must not be moving when calling this command!
  boolean sentcommand = false;
  while(!sentcommand){
    if (blueTooth.available())  {
      // Read the incomming signal
      char c = blueTooth.read();
      if (c == 'I') {
        // Send connection valid signal!
        blueTooth.println("-");
        sentcommand = true;
      }
    }else{
      delay(10);
    }
  }
}

void setupUnity(int timings[]){
  // Holds here till unity replies with a ready signal
  // Note* Bot must not be moving when calling this command!
  // --- Unity Timmings ---
  // - 0,1 [#.#f] (Time for a single straight block)
  // - 2,3 [#.#f] (Time for a 90 degree turn)
  // - 4,5 [#.#f] (Time for a 180 degree turn)
  boolean sentcommand = false;
  while(!sentcommand){
    if (blueTooth.available())  {
      // Read the incomming signal
      char c = blueTooth.read();
      if (c == 'C') {
        // Send Timming Initalize Command
        String command = "";
        command = command + timings[0] + timings[1] + timings[2] + timings[3] + timings[4] + timings[5];
        blueTooth.println(command);
        sentcommand = true;
      }
    }else{
      delay(10);
    }
  }
}

void unityUpdate(String command){
  // Waits for unity to send request command, then responds with next action
  // Note* Bot must not be moving when calling this command!
  //  --- Unity Commands: ----
  //  - S# (Continue straight for # number of intersections)
  //  - TR (Turn right 90 degrees)
  //  - TL (Turn Left 90 degrees)
  //  - TA (Turn Around 180 degrees)
  boolean sentcommand = false;
  while(!sentcommand){
    if (blueTooth.available())  {
      // Read the incomming signal
      char c = blueTooth.read();
      if (c == 'R') {
        // Send Next "Move" Command
        blueTooth.println(command);
        sentcommand = true;
      }
    }else{
      delay(10);
    }
  }
}

String countTillNextTurn(MyRobot mazeBot){
  // Gets squares count to next intersection based on maze cordinates and whichway command

  // Make new instance of MazeBot
  // Start at same position
  // Simulate continued movement till turn required.
  MyRobot tempBot{mazeBot.dir, mazeBot.turn, mazeBot.row, mazeBot.col, mazeBot.room, 0x00};
  uint8_t squares = 0;
  // Save Whichway Counters
  uint8_t hold1 = physicalBot.routeN;
  uint8_t hold2 = physicalBot.routeI;
  
  while(tempBot.turn == 0x00 || squares == 0){

  /* Update dir by turing in the maze */
    tempBot.dir = turnInMaze(tempBot);
    
   /* Reference return variable */
    uint8_t *updateValues;
  
    /* Update col and row loaction */
    updateValues = stepInMaze(tempBot);
    tempBot.row += *(updateValues);
    tempBot.col += *(updateValues + 1);
  
    /* Update room and bees */
    updateValues = roomInMaze(tempBot);
    tempBot.room = *(updateValues);
    tempBot.bees = *(updateValues + 1);
  
    /* Determine if a turn is needed */
    tempBot.turn = whichWay(tempBot);
    squares++;
  }
  // Reset real bot to orginal
    physicalBot.routeN = hold1;
    physicalBot.routeI = hold2;
  String command = "S";
  command = command + squares;
  return command;
}

String getNextTurnAction(MyRobot mazeBot){
  // uses whichway command and determines turn direction
  // Read mazeBot.turn for current turn instuction
  if(mazeBot.turn == 0x01) // Turn right
       return "TR";
  else if(mazeBot.turn == 0x02) // Turn left
       return "TL";
  else if(mazeBot.turn == 0x03) // Turn Around
       return "TA";
return "S0"; // error don't move anywhere?
}
